module net_dms3
implicit none
!---------------------------------------------
! number of networks for average
integer, parameter :: nfits=3
! number of hidden layers
integer, parameter :: max_nhid=2
! number of the summation of every layer's outputs
integer, parameter :: max_ny=500
! dimension of input data
integer, parameter :: max_ndim=100
! number of weights
integer, parameter :: max_nw=8000
!---------------------------------------------
integer :: init_flag_dms=0
integer :: nhid(nfits)
integer :: nl(0:max_nhid+1,nfits)
integer :: ny(nfits)
integer :: nw(nfits)
integer :: idyf(2,max_ny,nfits),idwf(2,max_ny,nfits)
real(kind=8) :: minmax(2,max_ndim+1,nfits)
real(kind=8) :: w(max_nw,nfits)
end module net_dms3
subroutine net_read_dms3b
use net_dms3
implicit none
integer :: i,j,k,ii
character(len=40) :: icoef
character(len=100),parameter :: dir="../weights/dms3b_SO2_2H2O/"
init_flag_dms=1
idyf=0; idwf=0
do ii=1,nfits
  write(icoef,*) ii
  open(120,file=trim(adjustl(dir))//"dms3bSO4H4."//trim(adjustl(icoef)),status="old")
  !--------------------------------------------------------
  read(120,*) nhid(ii)
  if(nhid(ii).gt.max_nhid) stop "max_nhid..."
  ny(ii)=0
  do i=0,nhid(ii)
    read(120,*) nl(i,ii)
    ny(ii)=ny(ii)+nl(i,ii)+1
  end do
  if(nl(0,ii).gt.max_ndim) stop "max_ndim..."
  nl(nhid(ii)+1,ii)=1; ny(ii)=ny(ii)+1
  read(120,*) nw(ii)
  if(nw(ii).gt.max_nw) stop "max_nw..."
  do i=1,nl(0,ii)+1
    read(120,*) minmax(:,i,ii)
  end do
  k=nl(0,ii)+1
  do i=1,nhid(ii)+1
    do j=1,nl(i,ii)
      k=k+1
      if(k.gt.max_ny) stop "max_ny..."
      read(120,*) idyf(1:2,k,ii),idwf(1:2,k,ii)
    end do
    k=k+1
  end do
  do i=1,nw(ii)
    read(120,*) w(i,ii)
  end do
!--------------------------------------------------------
close(120)
enddo
end subroutine net_read_dms3b
subroutine forward_dms3b(flag,p,y)
use net_dms3
implicit none
integer,intent(in) :: flag
real(kind=8),intent(in) :: p(nl(0,1))
real(kind=8),intent(out) :: y(max_ny)
real(kind=8),external :: ddot
real(kind=8) :: v
integer :: i,j,k,nyx
integer :: inc
nyx=0
do i=0,nhid(flag)
  nyx=nyx+nl(i,flag)+1
  y(nyx)=1.d0
end do
do i=1,nl(0,flag)
  y(i)=2.d0*(p(i)-minmax(1,i,flag))/(minmax(2,i,flag)-minmax(1,i,flag))-1.d0
end do
k=nl(0,flag)+1
do i=1,nhid(flag)+1
  inc=nl(i-1,flag)+1
  do j=1,nl(i,flag)
    k=k+1
    v=ddot(inc,w(idwf(1,k,flag):idwf(2,k,flag),flag),1,y(idyf(1,k,flag):idyf(2,k,flag)),1)
    if(k.lt.ny(flag)) then
      y(k)=v/sqrt(1.d0+v**2)
    else if(k.eq.ny(flag)) then
      y(k)=v
    end if
  end do
  k=k+1
end do
y(ny(flag))=(y(ny(flag))+1.d0)*(minmax(2,nl(0,flag)+1,flag)-minmax(1,nl(0,flag)+1,flag))/2.d0+minmax(1,nl(0,flag)+1,flag)
end subroutine forward_dms3b
subroutine dipole3b_SO4H4(x,v)
! Order: S O O O O H H H H 
! Units: r in Angstrom, v in Debye
use net_dms3
implicit none
real(kind=8), intent(in) :: x(43,3)
real(kind=8), intent(out) :: v(3)
real(kind=8) :: y(max_ny)
integer :: i
if(init_flag_dms.eq.0) then
  call net_read_dms3b
end if
v=0.d0
call forward_dms3b(1,x(1:43,1),y)
v(1)=y(ny(1))
call forward_dms3b(2,x(1:42,2),y)
v(2)=y(ny(2))
call forward_dms3b(3,x(1:43,3),y)
v(3)=y(ny(3))
end subroutine dipole3b_SO4H4

subroutine dms3b_SO4H4(x,v)
! Order: S O O O O H H H H 
! Units: x in Angstrom, v in Debye
use net_dms3
implicit none
real(kind=8),intent(in) :: x(3,9)
real(kind=8),intent(out) :: v(3)
real(kind=8) :: xx(3,9),yy(3,9),zz(3,9),d(3),d2(3),cost,phi,rot(3,3),rot2(3,3),rt1(3,3),rt2(3,3),r(36)
real(kind=8) :: s(43,3),vtmp(3),vv(3)
real(kind=8),parameter :: ang=0.529177212d0
integer :: i,j,k,id,l
id=0
do j=1,8
 do k=j+1,9
  id=id+1
  r(id)=dsqrt(sum((x(:,j)-x(:,k))**2))
 end do
end do
xx(:,1)=0.d0
do i=2,9
  xx(:,i)=x(:,i)-x(:,1)
enddo

d=xx(:,4)
cost=d(3)/dsqrt(sum(d**2))
phi=dacos(d(1)/dsqrt(d(1)**2+d(2)**2))
if(d(2).lt.0.d0) then
  phi=-phi
endif
call rmat(-cost,-phi,0.d0,rot)
do i=1,9
  call dgemv('N',3,3,1.d0,rot,3,xx(1:3,i),1,0.d0,yy(1:3,i),1)
enddo

d2=yy(:,3)
phi=dacos(d2(1)/dsqrt(d2(1)**2+d2(2)**2))
if(d2(2).lt.0.d0) then
  phi=-phi
endif
call rmat(1.d0,-phi,0.d0,rot2)
do j=1,9
   call dgemv('N',3,3,1.d0,rot2,3,yy(1:3,j),1,0.d0,zz(1:3,j),1)
end do
s=0.d0


do j=1,3
   s(1:36,j)=r(1:36)
end do
s(37:38,1)=zz(1,2:3)
s(39:43,1)=zz(1,5:9)
s(37:38,3)=zz(3,2:3)
s(39:43,3)=zz(3,5:9)
s(37,2)=zz(2,2)
s(38:42,2)=zz(2,5:9)
 
call dipole3b_SO4H4(s,vtmp)
!----------------------------------------------------------------------
rt1(:,:)=0.d0
rt2(:,:)=0.d0
call nizhen(rot,rt1,3)
call nizhen(rot2,rt2,3)
call dgemv('N',3,3,1.d0,rt2,3,vtmp(1:3),1,0.d0,vv(1:3),1)
call dgemv('N',3,3,1.d0,rt1,3,vv(1:3),1,0.d0,v(1:3),1)
end subroutine dms3b_SO4H4

