module net_dms1
implicit none
!---------------------------------------------
! number of networks for average
integer, parameter :: nfits=2
! number of hidden layers
integer, parameter :: max_nhid=2
! number of the summation of every layer's outputs
integer, parameter :: max_ny=500
! dimension of input data
integer, parameter :: max_ndim=35
! number of weights
integer, parameter :: max_nw=5000
!---------------------------------------------
integer :: init_flag_dms=0
integer :: nhid(nfits)
integer :: nl(0:max_nhid+1,nfits)
integer :: ny(nfits)
integer :: nw(nfits)
integer :: idyf(2,max_ny,nfits),idwf(2,max_ny,nfits)
real(kind=8) :: minmax(2,max_ndim+1,nfits)
real(kind=8) :: w(max_nw,nfits)
end module net_dms1
subroutine net_read_dms
use net_dms1
implicit none
integer :: i,j,k,ii
character(len=40) :: icoef
character(len=100),parameter :: dir="../weights/dmsSO2/"
init_flag_dms=1
idyf=0; idwf=0
do ii=1,nfits
  write(icoef,*) ii
  open(120,file=trim(adjustl(dir))//"SO2dms."//trim(adjustl(icoef)),status="old")
  !--------------------------------------------------------
  read(120,*) nhid(ii)
  if(nhid(ii).gt.max_nhid) stop "max_nhid..."
  ny(ii)=0
  do i=0,nhid(ii)
    read(120,*) nl(i,ii)
    ny(ii)=ny(ii)+nl(i,ii)+1
  end do
  if(nl(0,ii).gt.max_ndim) stop "max_ndim..."
  nl(nhid(ii)+1,ii)=1; ny(ii)=ny(ii)+1
  read(120,*) nw(ii)
  if(nw(ii).gt.max_nw) stop "max_nw..."
  do i=1,nl(0,ii)+1
    read(120,*) minmax(:,i,ii)
  end do
  k=nl(0,ii)+1
  do i=1,nhid(ii)+1
    do j=1,nl(i,ii)
      k=k+1
      if(k.gt.max_ny) stop "max_ny..."
      read(120,*) idyf(1:2,k,ii),idwf(1:2,k,ii)
    end do
    k=k+1
  end do
  do i=1,nw(ii)
    read(120,*) w(i,ii)
  end do
!--------------------------------------------------------
close(120)
enddo
end subroutine net_read_dms
subroutine forward_dms(flag,p,y)
use net_dms1
implicit none
integer,intent(in) :: flag
real(kind=8),intent(in) :: p(nl(0,1))
real(kind=8),intent(out) :: y(max_ny)
real(kind=8),external :: ddot
real(kind=8) :: v
integer :: i,j,k,nyx
integer :: inc
nyx=0
do i=0,nhid(flag)
  nyx=nyx+nl(i,flag)+1
  y(nyx)=1.d0
end do
do i=1,nl(0,flag)
  y(i)=2.d0*(p(i)-minmax(1,i,flag))/(minmax(2,i,flag)-minmax(1,i,flag))-1.d0
end do
k=nl(0,flag)+1
do i=1,nhid(flag)+1
  inc=nl(i-1,flag)+1
  do j=1,nl(i,flag)
    k=k+1
    v=ddot(inc,w(idwf(1,k,flag):idwf(2,k,flag),flag),1,y(idyf(1,k,flag):idyf(2,k,flag)),1)
    if(k.lt.ny(flag)) then
      y(k)=v/sqrt(1.d0+v**2)
    else if(k.eq.ny(flag)) then
      y(k)=v
    end if
  end do
  k=k+1
end do
y(ny(flag))=(y(ny(flag))+1.d0)*(minmax(2,nl(0,flag)+1,flag)-minmax(1,nl(0,flag)+1,flag))/2.d0+minmax(1,nl(0,flag)+1,flag)
end subroutine forward_dms
subroutine dipole_SO2(x,v)
! Order: S O O  
! Units: r in Angstrom, v in Debye
use net_dms1
implicit none
real(kind=8), intent(in) :: x(5,2)
real(kind=8), intent(out) :: v(2)
real(kind=8) :: y(max_ny)
integer :: i
if(init_flag_dms.eq.0) then
  call net_read_dms
end if
v=0.d0
do i=1,nfits
  call forward_dms(i,x(1:4,i),y)
  v(i)=y(ny(i))
enddo
end subroutine dipole_SO2

subroutine dms_SO2(x,v)
! Order: S O O
! Units: x in Angstrom, v in Debye
use net_dms1
implicit none
real(kind=8),intent(in) :: x(3,3)
real(kind=8),intent(out) :: v(3)
real(kind=8) :: xx(3,3),yy(3,3),zz(3,3),d(3),d2(3),cost,phi,rot(3,3),rt(3,3),r(3),rot2(3,3),rt2(3,3)
real(kind=8) :: s(5,2),vtmp(3),vv(3),vvv(3)
real(kind=8),parameter :: ang=0.529177212d0
integer :: i,j,k,id,l
id=0
do j=1,2
 do k=j+1,3
  id=id+1
  r(id)=dsqrt(sum((x(:,j)-x(:,k))**2))
 end do
end do
!--------------------
xx(:,1)=0.d0
do i=2,3
  xx(:,i)=x(:,i)-x(:,1)
enddo
d=xx(:,2)
cost=d(3)/dsqrt(sum(d**2))
phi=dacos(d(1)/dsqrt(d(1)**2+d(2)**2))
if(d(2).lt.0.d0) then
  phi=-phi
end if
call rmat(-cost,-phi,0.d0,rot)
do i=1,3
  call dgemv('N',3,3,1.d0,rot,3,xx(1:3,i),1,0.d0,yy(1:3,i),1)
end do
!-------------------
d2=yy(:,3)
phi=dacos(d2(1)/dsqrt(d2(1)**2+d2(2)**2))
if(d2(2).lt.0.d0) then
  phi=-phi
end if
call rmat(1.d0,-phi,0.d0,rot2)
do j=1,3
   call dgemv('N',3,3,1.d0,rot2,3,yy(1:3,j),1,0.d0,zz(1:3,j),1)
end do


s=0.d0
do i=1,2
  s(1:3,i)=r(1:3)
enddo
s(4,1)=zz(1,3)
s(4,2)=zz(3,3)
call dipole_SO2(s,vtmp)
vv=0.d0
vv(1)=vtmp(1)
vv(2)=0.d0
vv(3)=vtmp(2)


!----------------------------------------------------------------------
rt(:,:)=0.d0
rt2(:,:)=0.d0
call nizhen(rot2,rt2,3)
call nizhen(rot,rt,3)

call dgemv('N',3,3,1.d0,rt2,3,vv(1:3),1,0.d0,vvv(1:3),1)
call dgemv('N',3,3,1.d0,rt,3,vvv(1:3),1,0.d0,v(1:3),1)
end subroutine dms_SO2

