module net2
implicit none
!---------------------------------------------
! number of networks for average
integer, parameter :: nfits=1
! number of hidden layers
integer, parameter :: max_nhid=2
! number of the summation of every layer's outputs
integer, parameter :: max_ny=500
! dimension of input data
integer, parameter :: max_ndim=59
! number of weights
integer, parameter :: max_nw=40000
!---------------------------------------------
integer :: init_flag=0
integer :: nhid(nfits)
integer :: nl(0:max_nhid+1,nfits)
integer :: ny(nfits)
integer :: nw(nfits)
integer :: idyf(2,max_ny,nfits),idwf(2,max_ny,nfits)
real(kind=8) :: minmax(2,max_ndim+1,nfits)
real(kind=8) :: w(max_nw,nfits)
end module net2
subroutine net2_read
use net2
implicit none
integer :: i,j,k,ii
character(len=40) :: icoef
character(len=100),parameter :: dir="../weights/pes2b_SO2_H2O/"
init_flag=1
idyf=0; idwf=0
do ii=1,nfits
  write(icoef,*) ii
  open(120,file=trim(adjustl(dir))//"SO3H2net."//trim(adjustl(icoef)),status="old")
  read(120,*) nhid(ii)
  if(nhid(ii).gt.max_nhid) stop "max_nhid..."
  ny(ii)=0
  do i=0,nhid(ii)
    read(120,*) nl(i,ii)
    ny(ii)=ny(ii)+nl(i,ii)+1
  end do
  if(nl(0,ii).gt.max_ndim) stop "max_ndim..."
  nl(nhid(ii)+1,ii)=1; ny(ii)=ny(ii)+1
  read(120,*) nw(ii)
  if(nw(ii).gt.max_nw) stop "max_nw..."
  do i=1,nl(0,ii)+1
    read(120,*) minmax(:,i,ii)
  end do
  k=nl(0,ii)+1
  do i=1,nhid(ii)+1
    do j=1,nl(i,ii)
      k=k+1
      if(k.gt.max_ny) stop "max_ny..."
      read(120,*) idyf(1:2,k,ii),idwf(1:2,k,ii)
    end do
    k=k+1
  end do
  do i=1,nw(ii)
    read(120,*) w(i,ii)
  end do
!--------------------------------------------------------
close(120)
enddo
end subroutine net2_read
subroutine forward(flag,p,y)
use net2
implicit none
integer,intent(in) :: flag
real(kind=8),intent(in) :: p(nl(0,1))
real(kind=8),intent(out) :: y(max_ny)
real(kind=8),external :: ddot
real(kind=8) :: v
integer :: i,j,k,nyx
integer :: inc
nyx=0
do i=0,nhid(flag)
  nyx=nyx+nl(i,flag)+1
  y(nyx)=1.d0
end do
do i=1,nl(0,flag)
  y(i)=2.d0*(p(i)-minmax(1,i,flag))/(minmax(2,i,flag)-minmax(1,i,flag))-1.d0
end do
k=nl(0,flag)+1
do i=1,nhid(flag)+1
  inc=nl(i-1,flag)+1
  do j=1,nl(i,flag)
    k=k+1
    v=ddot(inc,w(idwf(1,k,flag):idwf(2,k,flag),flag),1,y(idyf(1,k,flag):idyf(2,k,flag)),1)
    if(k.lt.ny(flag)) then
      y(k)=v/sqrt(1.d0+v**2)
    else if(k.eq.ny(flag)) then
      y(k)=v
    end if
  end do
  k=k+1
end do
y(ny(flag))=(y(ny(flag))+1.d0)*(minmax(2,nl(0,flag)+1,flag)-minmax(1,nl(0,flag)+1,flag))/2.d0+minmax(1,nl(0,flag)+1,flag)
end subroutine forward
subroutine energy2b_SO3H2(x,v)
use net2
implicit none
real(kind=8), intent(in) :: x(max_ndim)
real(kind=8), intent(out) :: v
real(kind=8) :: y(max_ny)
integer :: i
if(init_flag.eq.0) then
  call net2_read
end if
v=0.d0
do i=1,nfits
  call forward(i,x,y)
  v=v+y(ny(i))
enddo
v=v/dble(nfits)
end subroutine energy2b_SO3H2

subroutine pes2b_SO3H2(r,v)
! Order: S O O O H H  
! Units: r in Angstrom, v in eV
implicit none
real(kind=8),intent(in) :: r(15)
real(kind=8),intent(out) :: v
real(kind=8) :: p(58)
integer :: flag
call fi_SO3H2(r,p)
call energy2b_SO3H2(p,v)
v=v/1000.d0
end subroutine pes2b_SO3H2   

function f2b(x)
! Order: S O O O H H 
! Units: x in Angstrom, v in eV
implicit none
real(kind=8),intent(in) :: x(3,6)
real(kind=8) :: f2b,r(15)
real(kind=8),parameter :: ang=0.529177212d0
integer :: id,j,k
id=0
do j=1,5
 do k=j+1,6
  id=id+1
  r(id)=sqrt(sum((x(:,j)-x(:,k))**2))
  r(id)=1.d0/r(id)
 end do
end do
call pes2b_SO3H2(r,f2b)
end function f2b
